#ifdef USE_LCCD

#ifndef GAINCORRECTORPROCESSOR_H
#define GAINCORRECTORPROCESSOR_H 1

// MarlinTPC
#include "ChannelCorrectionHandler.h"

// Marlin
#include <marlin/Processor.h>

// LCIO
#include <lcio.h>

// stl
#include <string>

namespace marlintpc {

/** GainCorrectorProcessor: converts FADC counts into primary electrons (assumes a linear gain needing no corrections)
 *    @author J. Abernathy (UVic)<br>
 */
class GainCorrectorProcessor : public marlin::Processor {

public:

	GainCorrectorProcessor() ;

	virtual void init() ;

	virtual Processor*  newProcessor() { return new GainCorrectorProcessor(); }

	virtual void processRunHeader( lcio::LCRunHeader* run );

	virtual void processEvent( lcio::LCEvent * evt );

	virtual void check( lcio::LCEvent * evt );

	virtual void end();

private:

	/** the name of the collection the input pulses will be stored under
	*/
	std::string _input_tracker_pulses_collection_name;

	/** the name of the collection the output pulses will be stored under
	*/
	std::string _output_tracker_pulses_collection_name;

	/** the name of the collection the channel corrections are stored with
	*/
	std::string _channel_corrections_collection_name;

	/** determines whether the output of this processor will be stored in a file after program termination (false) or not (true - default)
	*/
	bool _output_is_transient;

	// how many runs / events have been processed sofar
	int _number_of_runs;
	int _number_of_events;

	/** the handler for the channel corrections
	*/
	ChannelCorrectionHandler *_channel_corrections;
};

}

#endif // GAINCORRECTORPROCESSOR_H
#endif // USE_LCCD
