#ifndef TPCCONDITIONSHANDLER_H
#define TPCCONDITIONSHANDLER_H

// tpcconddata
#include <TPCConditions.h>

// stl
#include <map>

// lccd
#include "lccd/IConditionsChangeListener.hh"

namespace marlintpc {

/**
 *	TPCConditionsHandler: A handler class for keeping track of TPC conditions during run-time
 *
 *  @author Jason Abernathy, University of Victoria
 */
class TPCConditionsHandler: public lccd::IConditionsChangeListener {

public:

	/** Constructor for use with lccd
	*
	* @param collection_name the name of the collection registered with lccd in the conditions processor
	*/
	TPCConditionsHandler( std::string collection_name );

	/** Destructor
	*
	*/
	~TPCConditionsHandler();

	/**
	* return the drift velocity (in mm/us)
	*/
	float getDriftVelocity() const;

	/**
	* return the diffusion coefficent for the longitudinal diffusion (in sqrt(mm))
	*/
	float getLongDiffusionCoef() const;

	/**
	* return the defocussion constant for the longitudinal diffusion (in mm)
	*/
	float getLongDefocussing() const;

	/**
	* return the diffusion coefficent for the transversal diffusion (in sqrt(mm))
	*/
	float getTransDiffusionCoef() const;

	/**
	* return the defocussion constant for the transversal diffusion (in mm)
	*/
	float getTransDefocussing() const;

	/**
	* return the amplification of the gas amplification device (e.g. GEMs)
	*/
	float getAmplification() const;

	/**
	* Called by LCCD whenever the conditions change
	*
	*/
	void conditionsChanged( EVENT::LCCollection* col );

private:

	tpcconddata::TPCConditions _tpc_conditions;

};

} // namespace marlintpc

#endif // CHANNELCORRECTIONHANDLER_H
