#ifndef TRACK_FITTER_SIMPLE_MINIMIZER_H
#define TRACK_FITTER_SIMPLE_MINIMIZER_H

#include <string>
#include <marlin/StringParameters.h>
#include <EVENT/LCRunHeader.h>
#include <EVENT/Track.h>
#include <IMPL/TrackImpl.h>
#include <EVENT/TrackerHit.h>
#include "TrackFitterBase.h"

namespace gear
{
    class TPCParameters;
    class PadRowLayout2D;
}//namespace gear

namespace marlintpc
{
    /** Simple \f$\chi^2\f$-like fitter which sets all errors to 1.
     *  The value of the sum of the squared residuals is not useful to estimate the errors,
     *  but the minimisation should work.
     *
     *  The CalculateResiduals() function from TrackFitterBase is used.
     */

    class TrackFitterSimpleMinimizer : public TrackFitterBase
    {
      public:
      
	TrackFitterSimpleMinimizer();

	virtual ~TrackFitterSimpleMinimizer(){}

	/// Well defined interface to create a new instance of the track fitter
	static TrackFitterBase * getInstance(LCParameters const * parameters);

	virtual std::string getRevision() const;

	/// Information what fitter implementation this is.
	virtual unsigned char getFitterType() const;

	/// The implementation using the fitter function with all errors set to 1.
	virtual IMPL::TrackImpl *fitTrack(EVENT::Track const * seedTrack) const;

      protected:
    };

}// namespace marlintpc

#endif // TRACK_FITTER_SIMPLE_MINIMIZER_H
