#ifdef USE_TRACK_FITTER_KALMAN
#include "HeliKalTrack.h"
#include "TPCHitMap.h"

//GEAR
#include "gear/GEAR.h"
#include "gear/TPCParameters.h"
#include "gear/PadRowLayout2D.h"
#include "gearimpl/TPCModuleImpl.h"
#include "gearxml/GearXML.h"

#include <marlin/Global.h>
#include <streamlog/streamlog.h>

//LCIO
#include "UTIL/Operators.h"

//KalTest
#include "kaltest/TKalTrackSite.h"
#include "kaltest/TKalTrackState.h"
#include "kaltest/TVKalState.h"
#include "kaltest/TKalDetCradle.h"
#include "kaltest/THelicalTrack.h"

//KalDetector, MeasLayer and KalHit
#include "kaldet/EXTPCKalDetector.h"
#include "kaldet/EXTPCMeasLayer.h"
#include "kaldet/EXTPCHit.h"

#include "TVirtualPad.h"
#include "TPolyMarker3D.h"
#include "TMath.h"

#include <fstream>

using namespace std;
namespace marlintpc {

  /** helper function to restrict the range of the azimuthal angle to ]-pi,pi]*/
  inline double toBaseRange( double phi){
    while( phi <= -M_PI ){  phi += 2. * M_PI ; }
    while( phi >   M_PI ){  phi -= 2. * M_PI ; }
    return phi ;
  }
  

HeliKalTrack::HeliKalTrack(const TPCParameters* tp)
:AttKalTrack(tp)
{
  SetOwner();
}

HeliKalTrack::HeliKalTrack(const HeliKalTrack& orig)
        : AttKalTrack(orig), TKalTrack(orig)
{
  //TKalTrack
  SetOwner(false);
}

AttKalTrack* HeliKalTrack::create(const AttKalTrack * orig)
{
  //if (!orig) return new HeliKalTrack;
     
  const HeliKalTrack * ptr = dynamic_cast<const HeliKalTrack *>(orig);
  if (ptr) 
     return new HeliKalTrack(*ptr);
  else     
     return 0;
}

void HeliKalTrack::reset()
{
  //TKalTrack
  SetOwner();
  Clear();

  _hits.clear();

  HeliKalTrack hel(_tpcParameters);
  hel._domain = _domain;
  hel._tpcHitMap = _tpcHitMap;
  hel._minTrackHits=_minTrackHits;
  hel._maxSkipRows =_maxSkipRows;

  *this = hel;
}

TMatrixD HeliKalTrack::getStateVec()
{
  TMatrixD m(kSdim,1);
  return m;
}

bool HeliKalTrack::searchTrack(ModuleRowHit firstHit)
{
  ////////FIXME//////////
  const double MAX_DELTA=2.;
  ////////FIXME//////////


  streamlog_out(DEBUG3) << "------------------------------------" << endl
                        << "In HeliKalTrack::SearchTrack()"       << endl
                        << "------------------------------------" << endl;

  bool foundTrack=false;

  _domain=new TPCHitDomain(_tpcParameters,_maxSkipRows);
  if(!_domain->setDomain(firstHit,1))return false;//next hit domain

  const double *pos=TPCHitMap::getHitPosition(firstHit);

  list<ModuleRowHit> hitList1=_tpcHitMap->getHitsInDomain(*_domain);
  list<ModuleRowHit>::iterator secondHit;

  for(secondHit=hitList1.begin(); secondHit!=hitList1.end(); secondHit++)
  {
     if(foundTrack)break;
  
     //hit position selection hit1 and hit2
     const double* pos2   =TPCHitMap::getHitPosition(*secondHit);
     double delta12=fabs( pos[1]-pos2[1] );
     if(delta12>MAX_DELTA)continue;
  
     if(!_domain->setDomain(*secondHit,1))break;

     list<ModuleRowHit> hitList2=_tpcHitMap->getHitsInDomain(*_domain);
     list<ModuleRowHit>::iterator thirdHit;
    
     for(thirdHit=hitList2.begin();thirdHit!=hitList2.end(); thirdHit++)
     {
       if(foundTrack)break;

       //hit position selection for hit2 and hit3
       const double* pos3   =TPCHitMap::getHitPosition(*thirdHit);
       double delta23=fabs( pos2[1]-pos3[1] );
       if(delta23>MAX_DELTA)continue;

       //reset track
       reset();

       doAddAndFilter(firstHit);            //add the 1st hit
       if(!doAddAndFilter(*secondHit))break;//add the 2nd hit  
       if(!doAddAndFilter(*thirdHit) )break;//add the 3rd hit
  
       if(!_domain->setDomain(*thirdHit,1))return false;
       searchHit();//it's recursive

       if( getNumberOfHits()>= _minTrackHits )foundTrack=true;
     }
  }

  delete _domain;
  return foundTrack;
}


bool HeliKalTrack::doAddAndFilter(ModuleRowHit hit)
{
    ///////FIXME/////////////
    #ifdef OLD_UNIT_SYSTEM
    const double kmm2cm=0.1;
   #else
    const double kmm2cm=1.;
   #endif
    const double DELTA_TANL=3;
    const double MAX_DELTA=2.;
    ///////FIXME/////////////

    const double* hitPos=TPCHitMap::getHitPosition(hit);
    TVector3 xv(hitPos[0]*kmm2cm, hitPos[1]*kmm2cm, hitPos[2]*kmm2cm);

    if( getNumberOfHits()>=1 && getNumberOfHits()<3 )
    {
      const double* lastHitPos=TPCHitMap::getHitPosition(_lastBestHit);
      double delta=fabs(hitPos[1]-lastHitPos[1]);
      if(delta>MAX_DELTA)
      {
         streamlog_out(DEBUG3) << "hitpos=" << hitPos[1] << "   lasthitpos=" << lastHitPos[1] << endl;
         return false;
      }
    }

    if(getNumberOfHits()==2){
       list<ModuleRowHit>& hitList = getHitsList();
       ModuleRowHit hit1 = *( hitList.begin()  );
       ModuleRowHit hit2 = *( hitList.rbegin() );

       const double* hitPos1=TPCHitMap::getHitPosition(hit1);
       const double* hitPos2=TPCHitMap::getHitPosition(hit2);

       double dr12=sqrt(pow(hitPos1[0]-hitPos2[0],2)+pow(hitPos1[1]-hitPos2[1],2));
       double dz12=hitPos2[2]-hitPos1[2];
       double tanl12=dz12/dr12;

       double dr23=sqrt(pow(hitPos[0]-hitPos2[0],2)+pow(hitPos[1]-hitPos2[1],2));
       double dz23=hitPos[2]-hitPos2[2];
       double tanl23=dz23/dr23;

       if(abs(tanl12-tanl23)>DELTA_TANL)return false;
     }      
                
    TrackerHit* tHit=TPCHitMap::getTrackerHit(hit);
    int module=TPCHitMap::getModule(hit);
    int row   =TPCHitMap::getRow(hit);

    //FIXME
    const int NROW=28;
    int layerserial  = ( (module < 2) ? 0 : (module < 5 ? 1 : 2) )*NROW + row;

    EXTPCKalDetector * kaldetPtr  = EXTPCKalDetector::GetInstance();
    EXTPCMeasLayer * measlayerPtr = static_cast<EXTPCMeasLayer *>(kaldetPtr->At(layerserial));

    double   meas [2];//
    double   dmeas[2];//

#if 1
    int      side = +1;
#else
    int      side = -1;
#endif

    TKalMatrix mv   = measlayerPtr->XvToMv(xv, side);
    double   d    = mv(1, 0);

    meas [0] = mv(0, 0);
    meas [1] = d;

    dmeas[0] = measlayerPtr->GetSigmaX(d);
    dmeas[1] = measlayerPtr->GetSigmaZ(d);

    double b = EXTPCKalDetector::GetBfield();
    double v = EXTPCKalDetector::GetVdrift();


    streamlog_out(DEBUG3) << "layerserial=" << layerserial << " : (module,row)=(" << module << "," << row << ")" << endl
                          << "xv=" << xv.X() << " " << xv.Y() << " " << xv.Z() << endl
                          << "meas [0]=" << meas[0] << " meas[1]=" << meas[1] << endl;

    EXTPCHit *hp = new EXTPCHit(*measlayerPtr,meas,dmeas,side,v, static_cast<void *>(tHit),b);

    //Set the init helix
    if(getNumberOfHits()==0)
    {
      if (GetEntries()) Clear();
      //Create a dummy site: sited
      TVTrackHit *htdPtr = new EXTPCHit(*static_cast<EXTPCHit *>(hp));
      TVTrackHit &hitd   = *htdPtr;

      hitd(0,1) = 1.e6;    // give a huge error to d
      hitd(1,1) = 1.e6;    // give a huge error to z

      TKalTrackSite &sited = *new TKalTrackSite(hitd);
      sited.SetHitOwner(); // site owns hit
      sited.SetOwner();    // site owns states 

      //Set dummy state to sited
      static TKalMatrix svd(kSdim,1);



      //fg: here we define the direction in which the particle 
      //    goes through the helix

#if 0
      svd(0,0) = 0.; 			// dr
      svd(1,0) = -3.1415926/2;		// phi0
      svd(2,0) = -0.3; 			// kappa
      svd(3,0) = 0.;			// dz
      svd(4,0) = 0;			// tan(lambda)
#else
      svd(0,0) = 0.; 			// dr
      svd(1,0) = -3.1415926/2;		// phi0
      svd(2,0) = +0.3; 			// kappa
      svd(3,0) = 0.;			// dz
      svd(4,0) = 0;			// tan(lambda)
#endif

      if (kSdim == 6) svd(5,0) = 0.; 	// t0

      static TKalMatrix C(kSdim, kSdim);
      for (Int_t i = 0; i < kSdim; i++) {
        C(i,i) = 1.e6;   // dummy error matrix
      }

      sited.Add(new TKalTrackState(svd, C, sited, TVKalSite::kPredicted));
      sited.Add(new TKalTrackState(svd, C, sited, TVKalSite::kFiltered));

      Add(&sited);
    }

    /////////////FIXME///////////////
    const double MAX_DELTACHI2 = 600 ;
    /////////////FIXME///////////////

    TKalTrackSite &site = *new TKalTrackSite(*hp);
    site.SetHitOwner();
    
    if(getNumberOfHits()<3)
    {
      //for 3 initial hits
      TKalTrack::AddAndFilter(site);
      list<ModuleRowHit>& hitlist = getHitsList();
      hitlist.push_back(hit);
      _lastBestHit=hit;
      return true;
    }
    else
    {
      if(TKalTrack::AddAndFilter(site) && GetCurSite().GetDeltaChi2()< MAX_DELTACHI2)
      {
        streamlog_out(DEBUG3) << "good: deltaChi2=" << GetCurSite().GetDeltaChi2() << endl;
        return true;
      }
      else
      {
        streamlog_out(DEBUG3) << "fail: deltaChi2=" << GetCurSite().GetDeltaChi2() << endl;
        delete &site;
        return false;
      }
    }

}


void HeliKalTrack::printStateVector()
{
  streamlog_out(MESSAGE4) << "dr=" << GetCurSite().GetCurState()[0][0] 
       			  << " phi0=" << GetCurSite().GetCurState()[1][0]
       			  << " kappa=" << GetCurSite().GetCurState()[2][0]
       			  << " dz=" << GetCurSite().GetCurState()[3][0]
       			  << " tanl=" << GetCurSite().GetCurState()[4][0]
       			  << endl
       			  << " : (ndf,chi2)=(" << TKalTrack::GetNDF() 
       			  << " ," <<GetChi2() << ")"
       			  << endl;
}


double HeliKalTrack::getKappa()
{
  //TKalTrack
  return GetCurSite().GetCurState()[2][0];
}

double HeliKalTrack::getDeltaChi2()
{
  //TKalTrack
  return GetCurSite().GetDeltaChi2();
}

        

  void HeliKalTrack::toLCIOTrack( IMPL::TrackImpl* trk){
    
    TKalTrack& kaltrack = *this ; // code copied from Clupatra
    
    //fg: smooth back to the beginning - choose site 4, as the the first 3
    //    sites are probably not very reliable
    kaltrack.SmoothBackTo( 4 ) ;
  
    TKalTrackState& trkState = (TKalTrackState&) GetCurSite().GetCurState() ;

    TVKalSite &cursite = kaltrack.GetCurSite();
    
    Int_t    ndf  = kaltrack.GetNDF();
    Double_t chi2 = kaltrack.GetChi2();
    Double_t cl   = TMath::Prob(chi2, ndf);
    

    //============== convert parameters to LCIO convention ====
  
    THelicalTrack helix = trkState.GetHelix() ;
    
    
    // get the 5x5 sub matrix of the covariance matrix  
    const TMatrixD& c0 =  trkState.GetCovMat() ;
    TMatrixD covK(5,5) ;  for(int i=0;i<5;++i) for(int j=0;j<5;++j) covK[i][j] = c0[i][j] ;
    

    //  --- uncomment to get parameters with reference point at origin - typically `not for tbeam...
    // double dPhi ;
    // helix.MoveTo(  TVector3( 0., 0., 0. ) , dPhi , 0 , &covK ) ;
    
    //-------------------------------------
    
    
    double phi       =    toBaseRange( helix.GetPhi0() + M_PI/2. ) ;
    double omega     =    1. /helix.GetRho()  ;              
    double d0        =  - helix.GetDrho() ; 
    double z0        =    helix.GetDz()   ;
    double tanLambda =    helix.GetTanLambda()  ;
    
    //covariance matrix in LCIO - stored as lower triangle matrix where the order of parameters is: 
    // d0, phi, omega, z0, tan(lambda). So we have cov(d0,d0), cov( phi, d0 ), cov( phi, phi), ...
    
    
    // get alpha vor conversion of covariance parameters
    Double_t cpa  = trkState(2, 0);
    double alpha = omega / cpa  ;
    
    
    EVENT::FloatVec cov( 15 )  ; 
    cov[ 0] =   covK( 0 , 0 )   ; //   d0,   d0
    
    cov[ 1] = - covK( 1 , 0 )   ; //   phi0, d0
    cov[ 2] =   covK( 1 , 1 )   ; //   phi0, phi
    
    cov[ 3] = - covK( 2 , 0 ) * alpha   ; //   omega, d0
    cov[ 4] =   covK( 2 , 1 ) * alpha   ; //   omega, phi
    cov[ 5] =   covK( 2 , 2 ) * alpha * alpha  ; //   omega, omega
    
    cov[ 6] = - covK( 3 , 0 )   ; //   z0  , d0
    cov[ 7] =   covK( 3 , 1 )   ; //   z0  , phi
    cov[ 8] =   covK( 3 , 2 ) * alpha   ; //   z0  , omega
    cov[ 9] =   covK( 3 , 3 )   ; //   z0  , z0
    
    cov[10] = - covK( 4 , 0 )   ; //   tanl, d0
    cov[11] =   covK( 4 , 1 )   ; //   tanl, phi
    cov[12] =   covK( 4 , 2 ) * alpha  ; //   tanl, omega
    cov[13] =   covK( 4 , 3 )   ; //   tanl, z0
    cov[14] =   covK( 4 , 4 )   ; //   tanl, tanl
        
    trk->setD0( d0 ) ;  
    trk->setPhi( phi  ) ; // fi0  - M_PI/2.  ) ;  
    trk->setOmega( omega  ) ;
    trk->setZ0( z0  ) ;  
    trk->setTanLambda( tanLambda ) ;  
    
    trk->subdetectorHitNumbers().push_back( getNumberOfHits() ) ;  
    trk->subdetectorHitNumbers().push_back( getNumberOfHits() ) ;
    
    trk->setChi2( chi2 ) ;
    
    trk->setCovMatrix( cov ) ;
    
    float pivot[3] ;
    pivot[0] =  ((TKalTrackSite&) cursite).GetPivot()(0) ;
    pivot[1] =  ((TKalTrackSite&) cursite).GetPivot()(1) ;
    pivot[2] =  ((TKalTrackSite&) cursite).GetPivot()(2) ;
    
    trk->setReferencePoint( pivot ) ;
    
    streamlog_out( DEBUG ) << " kaltest track parameters: "
			   << " chi2/ndf " << chi2 / ndf  
			   << " chi2 " <<  chi2 << std::endl 
			   << " conv. level " << cl  
      
			   << "\t D0 "          <<  d0         <<  "[+/-" << sqrt( cov[0] ) << "] " 
			   << "\t Phi :"        <<  phi        <<  "[+/-" << sqrt( cov[2] ) << "] " 
			   << "\t Omega "       <<  omega      <<  "[+/-" << sqrt( cov[5] ) << "] " 
			   << "\t Z0 "          <<  z0         <<  "[+/-" << sqrt( cov[9] ) << "] " 
			   << "\t tan(Lambda) " <<  tanLambda  <<  "[+/-" << sqrt( cov[14]) << "] " 
      
			   << "\t pivot : [" << pivot[0] << ", " << pivot[1] << ", "  << pivot[2] 
			   << " - r: " << std::sqrt( pivot[0]*pivot[0]+pivot[1]*pivot[1] ) << "]" 
			   << std::endl ;
    
    trk->setNdf( ndf ) ;
    
    
#if LCIO_VERSION_GE( 1 , 60 )
    streamlog_out( DEBUG4 ) << lcio::header( trk ) << std::endl ;
#else
    streamlog_out( DEBUG4 ) << lcio::header( *trk ) << std::endl ;
#endif
    
    streamlog_out( DEBUG4 ) << lcio::lcshort( (lcio::Track*)trk ) << std::endl ;
    
    
    
    
  }
  

}
#endif
