#ifdef USE_TRACK_FITTER_KALMAN
#include "TPCHitMap.h"
#include "TPCHitDomain.h"
#include <EVENT/LCCollection.h>
#include <gear/TPCModule.h>
#include <streamlog/streamlog.h>

namespace marlintpc
{

TPCHitMap::TPCHitMap(const TPCParameters* tpc, const string inputName, const map<int,int>* moduleOrder)
:_tpcParameters(tpc),_hitInputColName(inputName),_empty(true)
{
    _moduleOrder=moduleOrder;
    _curModule=-1000;

    int nMod=_moduleOrder->size();

    _hitList=new list<TrackerHit*>*[nMod];
    for(int m=0;m<nMod;m++)
    {
       int moduleID=_moduleOrder->find(m)->second;
       const TPCModule& module=_tpcParameters->getModule(moduleID);
       int nRow=module.getNRows();
       _hitList[m]=new list<TrackerHit*>[nRow];
    }
}


TPCHitMap::~TPCHitMap()
{
    for(unsigned i=0; i<_moduleOrder->size(); i++) delete [] _hitList[i];
    delete [] _hitList;
}


void TPCHitMap::update()
{
  _curModule=-1000;
}


list<ModuleRowHit> TPCHitMap::getHitsInDomain(TPCHitDomain &domain)
{
    list<ModuleRowHit> hitList;

    int mod=domain.getModule();
    int row=domain.getRow();

    list<TrackerHit*>* trackerHitList=&(_hitList[mod])[row];
    for(list<TrackerHit*>::iterator iter=trackerHitList->begin(); iter!=trackerHitList->end(); iter++)
       hitList.push_back( ModuleRowHit(pair<int,int>(mod,row), iter) );

    return hitList;
}


void TPCHitMap::removeHits(list<ModuleRowHit>* hitList)
{
    streamlog_out(DEBUG3) << "In RemoveHits()" << endl;
    list<ModuleRowHit>::iterator iter;
    for(iter=hitList->begin(); iter!=hitList->end(); iter++)
    {
      int mod=(iter->first).first;
      int row=(iter->first).second;

      list<TrackerHit*>::iterator h=iter->second;

      (_hitList[mod])[row].erase(h);

      if( (_hitList[mod])[row].size()==0 )
      { 
          _recorder.remove( pair<int,int>(mod,row) );
      }
    }
   
    if(_recorder.size()==0)_empty=true;
}


void TPCHitMap::removeHit(ModuleRowHit hit)
{
    int mod=(hit.first).first;
    int row=(hit.first).second;
    list<TrackerHit*>::iterator h=hit.second;

    (_hitList[mod])[row].erase(h);

    if( (_hitList[mod])[row].size()==0 )
    { 
          _recorder.remove( pair<int,int>(mod,row) );
    }

    if(_recorder.size()==0)_empty=true;
}


void TPCHitMap::clear()
{
    for(unsigned mod=0;mod<_moduleOrder->size();mod++)
    {
       int modID=_moduleOrder->find(mod)->second;
       const TPCModule& module=_tpcParameters->getModule(modID);
       for(int row=0;row<module.getNRows();row++) (_hitList[mod])[row].clear();
    }

    _empty=true;
    _recorder.clear();
} 


void TPCHitMap::fillEventHits(LCEvent* evt)
{
    LCCollection *inputHits=0;
    try{
       inputHits=evt->getCollection(_hitInputColName);
    }
    catch  (DataNotAvailableException &e) {
       return;
    }
  
    clear();
     
    //read input hits, and put data into hitlist 
    int hitNum=inputHits->getNumberOfElements();
    streamlog_out(DEBUG3) << "Hit number=" << hitNum << endl;
    for(int iHit=0;iHit<hitNum;iHit++)
    {
       TrackerHit* inputHit = dynamic_cast<TrackerHit *>( inputHits->getElementAt(iHit));
       const double* pos=inputHit->getPosition();

       streamlog_out(DEBUG3) << "hit" << iHit << ": pos=(" << pos[0] << "," << pos[1] << "," << pos[2] << ")" << endl;
       const TPCModule* tpcmodule=&_tpcParameters->getNearestModule(pos[0],pos[1]);
       int mod=tpcmodule->getModuleID();
       int pad=tpcmodule->getNearestPad(pos[0],pos[1]);
       int row=tpcmodule->getRowNumber(pad);
       streamlog_out(DEBUG3) << "mod=" << mod << "  row=" << row <<  endl;
       (_hitList[mod])[row].push_back(inputHit);
    }
    if(hitNum>0)_empty=false;

    //set recorder
    int nMod=_moduleOrder->size();
    for(int m=0;m<nMod;m++)
    {
       int modID=_moduleOrder->find(m)->second;
       const TPCModule& module=_tpcParameters->getModule(modID);
       int nRow=module.getNRows();
       for(int r=0;r<nRow;r++)
       {
           if( (_hitList[m])[r].size() > 0 )
           {  
             _recorder.push_back( pair<int,int>(m,r) );
           }
       }
    }
}


ModuleRowHit TPCHitMap::getFirstHit()
{
    pair<int,int> pairModRow=*(_recorder.begin());
    ModuleRowHit hit(  pair<int,int>(pairModRow.first,   pairModRow.second),   
                      (_hitList[pairModRow.first])[pairModRow.second].begin()   
                    );

    return hit;
}


const double* TPCHitMap::getHitPosition(ModuleRowHit hit)
{
    const TrackerHit* hitPtr = *(hit.second);
    if(hitPtr!=0)
      return hitPtr->getPosition();
    else 
      return 0;
}


TrackerHit* TPCHitMap::getTrackerHit(ModuleRowHit hit)
{
    return *(hit.second);
}

}
#endif
