#ifndef SIGNALCOMBINERPROCESSOR_H
#define SIGNALCOMBINERPROCESSOR_H 1

// Marlin
#include <marlin/Processor.h>

namespace EVENT {

	class LCRunHeader;
	class LCEvent;
}

namespace marlintpc {

/** SignalCombinerProcessor : This processor takes the individual shaped signals (could be more than one per pad) and combines them into one signal
 *
 *    @author J. Abernathy (UVic)<br>
 */
class SignalCombinerProcessor : public marlin::Processor {

public:
	/** constructor
	 */
	SignalCombinerProcessor();

	/** destructor
	 */
	virtual ~SignalCombinerProcessor();

	/** used by Marlin to create a new processor
	 */
	virtual Processor* newProcessor() { return new SignalCombinerProcessor() ; }

	/**
	 * Called at the begin of the job before anything is read.
	 * Use to initialize the processor, e.g. book histograms.
	 */
	virtual void init();

	/** Called for every run
	 */
	virtual void processRunHeader( EVENT::LCRunHeader* run );

	/** Called for every event - the working horse.
	 */
	virtual void processEvent( EVENT::LCEvent * evt );

	/** Called after the event has been processed
	 */
	virtual void check( EVENT::LCEvent * evt );

	/** Called after data processing for clean up.
	 */
	virtual void end();

	/** a helper class for managing signals
	 */
	struct DataTag {

		DataTag( int pad, int module ):
			pad_id(pad),
			module_id(module) {}

		bool operator < (const DataTag &tag) const {

			if( tag.pad_id != pad_id ) {

				return tag.pad_id < pad_id;
			}
			else {

				return tag.module_id < module_id;
			}
		}

		bool operator != (const DataTag &tag) {

			return (tag.pad_id != pad_id) || (tag.module_id != module_id);
		}

		int pad_id;
		int module_id;
	};

protected:

private:

	/** the name of the input trackerdata collection
	 */
	std::string _input_collection_name;

	/** the name of the output tracker data collection from this processor
	 */
	std::string _output_collection_name;

	/** the width of a signal sample (1/Hz) in ns
	 */
	double _time_resolution;

	/** takes the constant tracker data object and merges it onto the second one
	 */
	void combine_data( const EVENT::TrackerData &, IMPL::TrackerDataImpl &, const double );
};

} // namespace marlintpc

#endif // SIGNALCOMBINERPROCESSOR_H
