#ifndef CYLINDRICAL2DMAGNETICFIELD_H
#define CYLINDRICAL2DMAGNETICFIELD_H 1

// stl
#include <string>
#include <vector>

// CLHEP
#include <CLHEP/Matrix/Vector.h>
#include <CLHEP/Vector/ThreeVector.h>

// MarlinTPC
#include "MagneticField.h"

namespace EVENT {

	class LCCollection;
}

namespace marlintpc {

/** Cylindrical2DMagneticField : A magnetic field from a cylindrical 2d field map (r and z)
 *
 *    @author Christian Grefe <br>
 */
class Cylindrical2DMagneticField: public MagneticField {

public:
	/** constructor
	 */
	Cylindrical2DMagneticField();

	/** destructor
	 */
	virtual ~Cylindrical2DMagneticField();

	/** returns a new field... similar to how the Marlin processor registration works
	 */
	virtual Field *new_field( void );

	/** constructs the field from an LCCollection of data objects (could be anything... field bin values or just parameters)
	 */
	virtual bool construct( EVENT::LCCollection * );

	struct FieldMapEntry {

		float _Br;
		float _Bz;
	};

protected:

	/** gets the field value at a point given in coil coordinates
	 */
	CLHEP::Hep3Vector get_field_value_coil_coordinates( const CLHEP::Hep3Vector & );


private:

	/** the boundaries of the field map
	 */
	double _r_min;
	double _r_max;
	double _z_min;
	double _z_max;

	/** the number of bins
	 */
	int _r_bins;
	int _z_bins;

	/** the distance between 2 bins
	 */
	double _dr;
	double _dz;

	/** the entries of the field map (the field vector at different positions)
	 */
	std::vector<std::vector<FieldMapEntry> > _field_map;
};

} // namespace marlintpc

#endif // CYLINDRICAL2DMAGNETICFIELD_H
