#ifndef DIDZMAPMAGNETICFIELD_H
#define DIDZMAPMAGNETICFIELD_H 1

// stl
#include <string>
#include <vector>

// CLHEP

// MarlinTPC
#include "Field.h"

namespace EVENT {

	class LCCollection;
}

namespace marlintpc {

/** DIDZMapMagneticField : A magnetic field from a DID map (mapped in z)
 *
 *    @author J. Abernathy (UVic)<br>
 */
class DIDZMapMagneticField: public Field {

public:
	/** constructor
	 */
	DIDZMapMagneticField();

	/** destructor
	 */
	virtual ~DIDZMapMagneticField();

	/** returns a new field... similar to how the Marlin processor registration works
	 */
	virtual Field *new_field( void );

	/** constructs the field from an LCCollection of data objects (could be anything... field bin values or just parameters)
	 */
	virtual bool construct( EVENT::LCCollection * );

	/** gets the field value at a specified point
	 */
	CLHEP::Hep3Vector get_field_value( const CLHEP::Hep3Vector & );

	/** returns the typename of the field
	 */
	std::string get_typename();

	struct FieldMapEntry {

		double _z;
		double _Bx;
	};

protected:

private:

	/** the boundaries of the DID map
	 */
	double _r_min;
	double _r_max;

	/** the distance between z bins
	 */
	double _dz;

	/** the entries of the field map (the field vector at different positions)
	 */
	std::vector<FieldMapEntry> _field_map;
};

} // namespace marlintpc

#endif // SOLENOIDZMAPMAGNETICFIELD_H
