#ifndef UNITBIDIRECTIONALELECTRICFIELD
#define UNITBIDIRECTIONALELECTRICFIELD 1

// MarlinTPC
#include "Field.h"

// CLHEP
#include <CLHEP/Matrix/Vector.h>

// stl
#include <string>
#include <vector>

namespace LCIO {

	class LCRunHeader;
}

namespace EVENT {

	class LCCollection;
}

namespace marlintpc {

/** UnitBiDirectionalElectricField : An electric field which is +1Vin the +z direction when z > 0 and -1V in the -z direction when z < 0
 *
 *    @author J. Abernathy (UVic)<br>
 */
class UnitBiDirectionalElectricField: public Field {

public:
	/** constructor
	 */
	UnitBiDirectionalElectricField ( );

	/** destructor
	 */
	virtual ~UnitBiDirectionalElectricField();

	/** returns a new field... similar to how the Marlin processor registration works
	 */
	virtual Field *new_field( void );

	/** constructs the field from an LCCollection of data objects (could be anything... field bin values or just parameters)
	 */
	virtual bool construct( EVENT::LCCollection * );

	/** gets the field value at a specified point
	 */
	CLHEP::Hep3Vector get_field_value( const CLHEP::Hep3Vector & );

	/** returns the typename of the field
	 */
	std::string get_typename();

protected:

private:

};

} // namespace marlintpc

#endif // UNITBIDIRECTIONALELECTRICFIELD
