
// stl
#include <string>
#include <cassert>

// LCIO
#include <EVENT/LCCollection.h>
#include <EVENT/LCGenericObject.h>

// marlintpc
#include "IdealSolenoidField.h"

using namespace CLHEP;
using namespace EVENT;
using namespace std;

namespace marlintpc {

IdealSolenoidField anIdealSolenoidField;

/** constructor
 */
IdealSolenoidField::IdealSolenoidField():
	Field("IdealSolenoidField") {
}


/** destructor
 */
IdealSolenoidField::~IdealSolenoidField() {}


/** returns a new field... similar to how the Marlin processor registration works
*/
Field *IdealSolenoidField::new_field( void ) { return new IdealSolenoidField; }


/** constructs the field from an LCCollection of data objects (could be anything... field bin values or just parameters)
*/
bool IdealSolenoidField::construct( LCCollection *collection ) {

	// grab the bounds from the collection
	_r_min = collection->getParameters().getFloatVal("r_min");
	_r_max = collection->getParameters().getFloatVal("r_max");
	_z_max = collection->getParameters().getFloatVal("z_max");

	LCGenericObject *field_value = dynamic_cast<LCGenericObject*>( collection->getElementAt(0) );
	assert(field_value);

	_field_value = CLHEP::Hep3Vector(field_value->getDoubleVal(0), field_value->getDoubleVal(1), field_value->getDoubleVal(2) );

	return true;
}


/** gets the field value at a specified point
*/
CLHEP::Hep3Vector IdealSolenoidField::get_field_value( const CLHEP::Hep3Vector &position ) {

	CLHEP::Hep3Vector no_field(0., 0., 0.);

	if( fabs(position.z()) > _z_max ) {

		return no_field;
	}

	const double radius = position.r();

	if( (radius < _r_min) || (radius > _r_max) ) {

		return no_field;
	}

	return _field_value;
}

} // namespace marlintpc
