#include <stdexcept>
#include <iostream>
#include <cmath>
#include <complex>
using namespace std;

#include "curlerGenerator.h"

std::pair<double,double> curlerGenerator::calculateIntersectionAngles(double R)
{
    double K = r*r - R*R - M2;
    //cout << "K " <<K<< endl;

    double tmp1 = a /2./ M2 *K ;
    double tmp2 = std::sqrt( tmp1*tmp1 - K*K/4./M2 + b*b*R*R/M2);

    double x1= -tmp1 + tmp2;
    double x2= -tmp1 - tmp2;
    //cout << "x1 " <<x1<< endl;
    //cout << "x2 " <<x2<< endl;
    
    
    double tmp3=b /2./ M2 *K ;
    double tmp4 = std::sqrt( tmp3*tmp3 - K*K/4./M2 + a*a*R*R/M2);

    double y1= -tmp3 + tmp4;
    double y2= -tmp3 - tmp4;
    //cout << "y1 " <<y1<< endl;
    //cout << "y2 " <<y2<< endl;

    double point1[2], point2[2];
    point1[0]=x1;
    point2[0]=x2;
    // find correct pairing
    if  ( ( fabs(x1*x1 +y1*y1 - R*R) < 1e-6) // double should be precise to better than 1e-6
	  &&  ( fabs((x1-a)*(x1-a) +(y1-b)*(y1-b) - r*r) < 1e-6) )
    {
	point1[1]=y1;
	point2[1]=y2;
    }
    else // also test the other pairing 
	if ( ( fabs(x1*x1 +y2*y2 - R*R) < 1e-6) // double should be precise to better than 1e-6
	     &&  ( fabs((x1-a)*(x1-a) +(y2-b)*(y2-b) - r*r) < 1e-6) )
	{
	    point1[1]=y2;
	    point2[1]=y1;
	}
	else // there is something wrong , throw an exception
	{
	    std::cerr << "curlerGenerator::calculateIntersectionAngles: calculated point don't match with circles"
		 << std::endl;
	    throw std::logic_error("No intersection found");
	}
    
//    cout << point1[0] << "\t"<<point1[1] << "\t";
//    cout << point2[0] << "\t"<<point2[1] << endl;
    std::complex<double> point1c(point1[0],point1[1]);
    std::complex<double> point2c(point2[0],point2[1]);

    if (std::arg(point1c) < std::arg(point2c) )
	return std::pair<double,double>(std::arg(point1c),std::arg(point2c));
    else
	return std::pair<double,double>(std::arg(point2c),std::arg(point1c));
    
}

curlerGenerator::curlerGenerator(double _a, double _b, double _r)
    : a(_a), b(_b), r(_r)
{
    M2=a*a+b*b;
}

double curlerGenerator::curlerGenerator::getM()
{
    return std::sqrt(M2);
}

double curlerGenerator::curlerGenerator::getr()
{
    return r;
}

double curlerGenerator::curlerGenerator::geta()
{
    return a;
}

double curlerGenerator::curlerGenerator::getb()
{
    return b;
}

